/*
Firmware to manage the 'hidden' functions of a RTL8366CB
GBit switch.
(C) 2010 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//This file provides stdin and stdout over the UART to the rest of the
//program.

#include <stdio.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
#include <util/delay.h>

static int uart_putchar(char c, FILE *stream) {
    if (c=='\n') uart_putchar('\r', stream);
    while(!(UCSR0A&0x20)) ;
    UDR0=c;
    return 0;
}

static int uart_getchar(FILE *stream) {
	unsigned char c;
    while(!(UCSR0A&0x80)) ;
    c=UDR0;
	if (c=='\r') return '\n';
    return c;
}

static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
static FILE mystdin = FDEV_SETUP_STREAM(NULL, uart_getchar, _FDEV_SETUP_READ);

void stdoutInit(int ubr) {
    UCSR0B=0x18;
    UBRR0L=ubr&0xff;
    UBRR0H=ubr>>8;
    stdout = &mystdout;
	stdin = &mystdin;
}